<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="cfg">
    <xsl:output method="xml" omit-xml-declaration="yes"/>
    <xsl:variable name="VERSION">9</xsl:variable>

    <xsl:template match="cfg:replace[not(cfg:node/@value)]">
        <rename><xsl:copy-of select="@* | cfg:node/@*"/></rename>
    </xsl:template>
    <xsl:template match="cfg:insert[@as='parent']">
        <insert-parent><xsl:copy-of select="@*[not(local-name()='as')] | cfg:node/@*"/></insert-parent>
    </xsl:template>
    <xsl:template match="cfg:replace[cfg:node/@value] | cfg:insert[not(@as='parent')]">
        <xsl:copy><xsl:apply-templates select="@* | cfg:node"/></xsl:copy>
    </xsl:template>

    <xsl:template match="cfg:processor[@type='ReplaceProcessor' and not(cfg:parameter/@name='node_value')]">
        <xsl:copy><xsl:copy-of select="@*[local-name()!='type']"/>
            <xsl:attribute name="type">RenameProcessor</xsl:attribute>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='ReplaceProcessor' and cfg:parameter/@name='node_value']">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="cfg:*[not(self::cfg:parameter)] | text() | comment()"/>
            <xsl:apply-templates select="." mode="NODES"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="cfg:processor[@type='InsertProcessor' and cfg:parameter/@name='node_value']">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="cfg:*[not(self::cfg:parameter)] | text() | comment()"/>
            <xsl:apply-templates select="cfg:parameter[@name='destination_as']"/>
            <xsl:apply-templates select="." mode="NODES"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="cfg:replace/cfg:node[not(@name) and @value]">
        <xsl:attribute name="new-nodes"><xsl:value-of select="@value"/></xsl:attribute>
    </xsl:template>
    <xsl:template match="cfg:node[@name]">
        <xsl:variable name="prefix">
            <xsl:if test="@prefix"><xsl:value-of select="@prefix"/>:</xsl:if>
        </xsl:variable>
        <xsl:attribute name="new-nodes">
            <xsl:choose>
                <xsl:when test="parent::cfg:insert/@as='attribute'">new_attribute(</xsl:when>
                <xsl:otherwise>new_element(</xsl:otherwise>
            </xsl:choose>
            <xsl:text/>'<xsl:value-of select="$prefix"/><xsl:value-of select="@name"/>'<xsl:text/>
            <xsl:for-each select="@value">
                <xsl:text>, </xsl:text>
                <xsl:choose>
                    <xsl:when test="starts-with(.,'@')">string(<xsl:value-of select="."/>)</xsl:when>
                    <xsl:otherwise><xsl:value-of select="."/></xsl:otherwise>
                </xsl:choose>
            </xsl:for-each>
            <xsl:text>)</xsl:text>
        </xsl:attribute>
    </xsl:template>

    <xsl:template match="cfg:processor[@type='ReplaceProcessor' and not(cfg:parameter/@name='node_name')]" mode="NODES">
        <parameter name="nodes">
            <xsl:attribute name="eval">
                <xsl:for-each select="cfg:parameter[@name='node_value']">
                    <xsl:choose>
                        <xsl:when test="@eval"><xsl:value-of select="@eval"/></xsl:when>
                        <xsl:otherwise>'<xsl:value-of select="text()"/>'</xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
            </xsl:attribute>
        </parameter>
    </xsl:template>
    <xsl:template match="cfg:processor[cfg:parameter/@name='node_name']" mode="NODES">
        <parameter name="nodes">
            <xsl:variable name="prefix">
                <xsl:for-each select="cfg:parameter[@name='node_prefix']"><xsl:value-of select="text()"/>:</xsl:for-each>
            </xsl:variable>
            <xsl:attribute name="eval">
                <xsl:choose>
                    <xsl:when test="cfg:parameter[@name='destination_as']/text() = 'attribute'">new_attribute(</xsl:when>
                    <xsl:when test="contains(parent::cfg:processor[@type='ReplaceProcessor']/@match, '/@')">new_attribute(</xsl:when>
                    <xsl:otherwise>new_element(</xsl:otherwise>
                </xsl:choose>
                <xsl:for-each select="cfg:parameter[@name='node_name']">
                    <xsl:choose>
                        <xsl:when test="@eval">concat('<xsl:value-of select="$prefix"/>', <xsl:value-of select="@eval"/>)</xsl:when>
                        <xsl:otherwise>'<xsl:value-of select="$prefix"/><xsl:value-of select="text()"/>'</xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
                <xsl:for-each select="cfg:parameter[@name='node_value']">
                    <xsl:text>, </xsl:text>
                    <xsl:choose>
                        <xsl:when test="starts-with(@eval,'@')">string(<xsl:value-of select="@eval"/>)</xsl:when>
                        <xsl:when test="@eval"><xsl:value-of select="@eval"/></xsl:when>
                        <xsl:otherwise>'<xsl:value-of select="text()"/>'</xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
                <xsl:text>)</xsl:text>
            </xsl:attribute>
        </parameter>
    </xsl:template>

    <!-- common -->
    <xsl:template match="/cfg:config/@version">
        <xsl:attribute name="version">
            <xsl:value-of select="$VERSION"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@* | comment()" priority="-1">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*" priority="-1">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>
